%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed B:
% Dynamic OPF in Presence of Renewable Energy and Electric Vehicles 
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% A set of Montecarlo Simulations VS Wind Analytical Uncertainty Cost Function 

clc
close all

                    %% WIND PARAMETERS
                    
                    Ws=100; % Scheduled Power
                    Vi=5;           
                    Vr=15;          
                    Vo=45;          
                    Wr=150;         
                    a= 15;          
                    b= -75;         
                    sg= 15.9577;    
                    c=sqrt(2)*sg;   
                    k= 2;           

N=100000;         % The competition uses only 2000

Cu=300;          
Co=700;          

ii=1;
iii=1;

CO=[];
CU=[];
W=[];

%% A set of montecarlo simulations
tic  
for i=1:N
    v(i)=wblrnd(c,k); 
    if (v(i)<Vi)||(v(i)>Vo)
        W(i)=0;
    elseif v(i)<Vr && Vi<v(i)
        W(i)=a*v(i)+b;
    else
        W(i)=Wr;
    end
    
   
        if Ws>=W(i)
            CO(ii)=Co*(Ws-W(i));
            ii=ii+1;
        else
            CU(iii)=Cu*(W(i)-Ws);
            iii=iii+1;
        end
end
   figure(1)
 
   subplot(4,1,1)
   hist(v)
   grid on;
   title('Histograms','color','Black','fontsize',13);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Wind speed [m/s]','color','Black','fontsize',9);
   subplot(4,1,2)
   hist(W)
   grid on;
   %title('Power Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Power [MW]','color','Black','fontsize',9);
   subplot(4,1,3)
   hist(CU)
   grid on;
   %title('Cost due to underestimate Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Cost due to underestimate [$]','color','Black','fontsize',9);
   subplot(4,1,4)
   hist(CO)
   grid on;
   %title('Cost due to overestimate Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Cost due to overestimate [$]','color','Black','fontsize',9);
   
   xo=mean(CO);
   xu=mean(CU);
   toc
   
   figure(2)
   CUCO=[CU CO];
   hist(CUCO)
   title( 'UCF Histogram','color','Black','fontsize',12) , grid on;
   ylabel('Scenarios','color','Black','fontsize',10); 
   xlabel('Penalty cost (UCF) [$]','color','Black','fontsize',11);
   ExpectedUncertaintyCost_Montecarlo=mean(CUCO)
   
%% Analytical Caculation

d=(Wr-b)/(sqrt(2)*a*sg);
g=(Ws-b)/(sqrt(2)*a*sg);
Cua=((Cu/2)*(sqrt(2*pi)*a*sg*(erf(d)-erf(g))+2*(Ws-Wr)*exp(-d^2)))+(Cu*(Wr-Ws)*(exp(-((Vr^2)/(2*sg^2)))-exp(-((Vo^2)/(2*sg^2)))));

d=(-b)/(sqrt(2)*a*sg);
g=(Ws-b)/(sqrt(2)*a*sg);
Coa1=-((Co/2)*(sqrt(2*pi)*a*sg*(erf(g)-erf(d))));
Coa2=(Co*(Ws)*(1-exp(-((Vi^2)/(2*sg^2)))+exp(-((Vo^2)/(2*sg^2)))+exp(-((b^2)/(2*a^2*sg^2)))));

Coa=Coa1+Coa2;

ExpectedUncertaintyCost_Analytical=Cua+Coa